@@ -2,6 +2,8 @@
 import os
 import logging
 import warnings
+import platform
+import sys
 
 from math import floor
 import numpy as np
@@ -22,7 +24,7 @@ from . import sed_tools
 class init_tools(abc.ABC):
 
     def init_output_infrastructure(self):
-        """Initialize the output infrastructure (i.e., folder).
+        """Initialize the output infrastructure (folder and save lists).
 
         This method is the first called in the initialization of the
         `DeltaModel`, after the configuration variables have been imported.
@@ -36,6 +38,10 @@ class init_tools(abc.ABC):
             os.makedirs(self.prefix_abspath)
             assert os.path.isdir(self.prefix_abspath)  # validate dir created
 
+        self._save_fig_list = dict()  # dict of figure variables to save
+        self._save_var_list = dict()  # dict of variables to save
+        self._save_var_list['meta'] = dict()  # set up meta dict
+
     def init_logger(self):
         """Initialize a logger.
 
@@ -57,9 +63,16 @@ class init_tools(abc.ABC):
         # add handler to logger object
         self.logger.addHandler(fh)
 
-        _msg = 'Output log file initialized.'
+        _msg = 'Output log file initialized'
         self.log_info(_msg, verbosity=0)
 
+        # log attributes of the model and environment
+        self.log_info('pyDeltaRCM version {}'.format(self.__pyDeltaRCM_version__))  # log the pyDeltaRCM version
+        self.log_info('Python version {}'.format(sys.version),
+                      verbosity=0)  # log the python version
+        self.log_info('Platform: {}'.format(platform.platform()),
+                      verbosity=0)  # log the os
+
     def import_files(self, kwargs_dict={}):
         """Import the input files.
 
@@ -124,16 +137,41 @@ class init_tools(abc.ABC):
                 if type(user_dict[k]) in expected_type:
                     input_file_vars[k] = user_dict[k]
                 else:
-                    raise TypeError('Input for "{_k}" not of the right type '
-                                    'in yaml configuration file "{_file}". '
-                                    'Input type was "{_wastype}", '
-                                    'but needs to be "{_exptype}".'
-                                    .format(_k=str(k), _file=self.input_file,
-                                            _wastype=type(k).__name__,
-                                            _exptype=expected_type))
+                    raise TypeError(f'Input for {str(k)} not of the '
+                                    'right type in yaml configuration '
+                                    'file {self.input_file}. '
+                                    'Input type was {type(k).__name__}, '
+                                    'but needs to be {expected_type}.')
             else:
                 input_file_vars[k] = default_dict[k]['default']
 
+        # add custom subclass yaml parameters (yaml or defaults) to input vars
+        for k, v in self.subclass_parameters.items():
+            if k in input_file_vars:
+                warnings.warn(UserWarning(
+                    'Custom subclass parameter name is already a '
+                    'default yaml parameter of the model, '
+                    'custom parameter value will not be used.'
+                ))
+            elif k in user_dict:
+                # get expected types
+                if not type(v['type']) is list:
+                    expected_type = [eval(v['type'])]
+                else:
+                    expected_type = [eval(_v) for _v in v['type']]
+                # evaluate against expected types
+                if type(user_dict[k]) in expected_type:
+                    input_file_vars[k] = user_dict[k]
+                else:
+                    raise TypeError(f'Input for {str(k)} not of the '
+                                    'right type in yaml configuration '
+                                    'file {self.input_file}. '
+                                    'Input type was {type(k).__name__}, '
+                                    'but needs to be {expected_type}.')
+            else:
+                # set using default value
+                input_file_vars[k] = v['default']
+
         # save the input file as a hidden attr and grab needed value
         self._input_file_vars = input_file_vars
         self.out_dir = self._input_file_vars['out_dir']
@@ -154,14 +192,13 @@ class init_tools(abc.ABC):
         _msg = 'Setting up model configuration'
         self.log_info(_msg, verbosity=0)
 
+        _msg = f'Model type is: {self.__class__.__name__}'
+        self.log_info(_msg, verbosity=0)
+
         # process the input file to attributes of the model
         for k, v in list(self._input_file_vars.items()):
             setattr(self, k, v)
 
-        # if checkpoint_dt is the default value (None), then set it to save_dt
-        if self.checkpoint_dt is None:
-            self.checkpoint_dt = self._save_dt
-
         # write the input file values to the log
         if not self._resume_checkpoint:
             for k, v in list(self._input_file_vars.items()):
@@ -186,8 +223,67 @@ class init_tools(abc.ABC):
         _msg = 'Random seed is: %s ' % str(self._seed)
         self.log_info(_msg, verbosity=0)
 
-    def set_constants(self):
+    def create_other_variables(self):
+        """Model implementation variables.
 
+        Creates variables for model implementation, from specified boundary
+        condition variables. This method is run during initial model
+        instantition. Internally, this method calls :obj:`set_constants` and
+        :obj:`create_boundary_conditions`.
+
+        .. note::
+
+            It is usually not necessary to re-run this method if you change
+            boundary condition variables, and instead only re-run
+            :obj:`create_boundary_conditions`.
+        """
+        _msg = 'Setting other variables'
+        self.log_info(_msg, verbosity=1)
+
+        self.init_Np_water = self._Np_water
+        self.init_Np_sed = self._Np_sed
+
+        self.dx = float(self._dx)
+
+        self.theta_sand = self._coeff_theta_sand * self._theta_water
+        self.theta_mud = self._coeff_theta_mud * self._theta_water
+
+        self.U_dep_mud = self._coeff_U_dep_mud * self._u0
+        self.U_ero_sand = self._coeff_U_ero_sand * self._u0
+        self.U_ero_mud = self._coeff_U_ero_mud * self._u0
+
+        self.L = int(round(self._Length / self._dx))        # num cells in x
+        self.W = int(round(self._Width / self._dx))         # num cells in y
+
+        # cross-stream center of domain idx
+        self.CTR = floor(self.W / 2.) - 1
+        if self.CTR <= 1:
+            self.CTR = floor(self.W / 2.)
+
+        self.set_constants()
+
+        self.create_boundary_conditions()
+
+        self._save_any_grids = (self._save_eta_grids or
+                                self._save_depth_grids or
+                                self._save_stage_grids or
+                                self._save_discharge_grids or
+                                self._save_velocity_grids or
+                                self._save_sedflux_grids or
+                                self._save_sandfrac_grids or
+                                self._save_discharge_components or
+                                self._save_velocity_components)
+        if self._save_any_grids:  # always save metadata if saving grids
+            self._save_metadata = True
+        self._is_finalized = False
+
+    def set_constants(self):
+        """Set the model constants.
+
+        Configure constants, including coordinates and distances, as well as
+        environmental constants (gravity), and kernels for smoothing
+        topography.
+        """
         _msg = 'Setting model constants'
         self.log_info(_msg, verbosity=1)
 
@@ -231,62 +327,52 @@ class init_tools(abc.ABC):
                                  [1, 0, 1],
                                  [1, 1, 1]]).astype(np.int64)
 
-    def create_other_variables(self):
-        """Model implementation variables.
+    def create_boundary_conditions(self):
+        """Create model boundary conditions
 
-        Creates variables for model implementation, from specified boundary
-        condition variables. This method is run during initial model
-        instantition, but it is also run any time an inlet flow condition
-        variable is changed, including `channel_flow_velocity`,
-        `channel_width`, `channel_flow_depth`, and
-        `influx_sediment_concentration`.
+        This method is run during model initialization to determine the
+        boundary conditions based on the initial conditions specified as model
+        input.
+
+        However, the method also should be called when certain boundary
+        condition variables are updated (e.g., during some custom model runs).
+        For example, if some property of the inlet flow condition is changed,
+        you will *likely* want to re-run this method (e.g., `u0`), because
+        there are several other parameters that depend on the value of the
+        inlet flow velocity (e.g., `Qw0` and `Qs0`); of course, your
+        scientific question may choose to leave these parameters as they are
+        too (in which case you should *not* re-run this method). See
+        :doc:`/examples/updating_boundary_conditions` for more information.
+
+        .. note::
+
+            This method is automatically called for the "named" variables used
+            by the BMI wrapper (e.g., `channel_flow_velocity`,
+            `channel_width`, `channel_flow_depth`, and
+            `influx_sediment_concentration`., so you do not need to call it
+            again if you modify the model boundary conditions that way.
         """
-        _msg = 'Setting other variables'
-        self.log_info(_msg, verbosity=1)
-
-        self.init_Np_water = self._Np_water
-        self.init_Np_sed = self._Np_sed
-
-        self.dx = float(self._dx)
-
-        self.theta_sand = self._coeff_theta_sand * self._theta_water
-        self.theta_mud = self._coeff_theta_mud * self._theta_water
-
-        self.U_dep_mud = self._coeff_U_dep_mud * self._u0
-        self.U_ero_sand = self._coeff_U_ero_sand * self._u0
-        self.U_ero_mud = self._coeff_U_ero_mud * self._u0
-
-        self.L = int(round(self._Length / self._dx))        # num cells in x
-        self.W = int(round(self._Width / self._dx))         # num cells in y
-
         # inlet length and width
         self.L0 = max(
             1, min(int(round(self._L0_meters / self._dx)), self.L // 4))
         self.N0 = max(
             3, min(int(round(self._N0_meters / self._dx)), self.W // 4))
 
-        self.set_constants()
-
         self.u_max = 2.0 * self._u0  # maximum allowed flow velocity
         self.C0 = self._C0_percent * 1 / 100.  # sediment concentration
 
         # (m) critial depth to switch to "dry" node
         self.dry_depth = min(0.1, 0.1 * self._h0)
 
-        # cross-stream center of domain idx
-        self.CTR = floor(self.W / 2.) - 1
-        if self.CTR <= 1:
-            self.CTR = floor(self.W / 2.)
-
-        self.gamma = (self.g * self._S0 * self._dx /
-                      (self._u0**2))  # water weighting coeff
+        self.gamma = (self.g * self.S0 * self._dx /
+                      (self.u0**2))  # water weighting coeff
 
         # (m^3) reference volume, volume to fill cell to characteristic depth
         self.V0 = self.h0 * (self._dx**2)
-        self.Qw0 = self._u0 * self.h0 * self.N0 * self._dx    # const discharge
+        self.Qw0 = self.u0 * self.h0 * self.N0 * self._dx    # const discharge
 
         # at inlet
-        self.qw0 = self._u0 * self.h0  # water unit input discharge
+        self.qw0 = self.u0 * self.h0  # water unit input discharge
         self.Qp_water = self.Qw0 / self._Np_water  # volume each water parcel
         self.qs0 = self.qw0 * self.C0  # sed unit discharge
         self.dVs = 0.1 * self.N0**2 * self.V0  # total sed added per timestep
@@ -314,31 +400,26 @@ class init_tools(abc.ABC):
         self.diffusion_multiplier = (self._dt / self.N_crossdiff * self._alpha
                                      * 0.5 / self._dx**2)
 
-        self._save_any_grids = (self._save_eta_grids or
-                                self._save_depth_grids or
-                                self._save_stage_grids or
-                                self._save_discharge_grids or
-                                self._save_velocity_grids or
-                                self._save_sedflux_grids or
-                                self._save_discharge_components or
-                                self._save_velocity_components)
-        self._save_any_figs = (self._save_eta_figs or
-                               self._save_depth_figs or
-                               self._save_stage_figs or
-                               self._save_discharge_figs or
-                               self._save_velocity_figs or
-                               self._save_sedflux_figs)
-        if self._save_any_grids:  # always save metadata if saving grids
-            self._save_metadata = True
-        self._is_finalized = False
-
     def create_domain(self):
-        """
-        Creates the model domain
+        """Create the model domain.
+
+        This method initializes the model domain, including coordinate arrays,
+        gridded fields (eta, qw, etc.) and cell type information. The method
+        is called during initialization of the `DeltaModel`, and likely does
+        not need to be called again.
+
+        .. hint::
+
+            If you need to modify the model domain after it has been created,
+            it is probably safe to modify attributes directly, but take care
+            to ensure any dependent fields are also approrpriately changed.
         """
         _msg = 'Creating model domain'
         self.log_info(_msg, verbosity=1)
 
+        # resolve any boundary conditions
+        self._hb = self.hb or self.h0  # basin depth
+
         # ---- empty arrays ----
         self.x, self.y = np.meshgrid(np.arange(0, self.W),
                                      np.arange(0, self.L))
@@ -347,6 +428,7 @@ class init_tools(abc.ABC):
 
         self.cell_type = np.zeros((self.L, self.W), dtype=np.int64)
         self.eta = np.zeros((self.L, self.W), dtype=np.float32)
+        self.eta0 = np.copy(self.eta)  # establish eta0 copy
         self.stage = np.zeros((self.L, self.W), dtype=np.float32)
         self.depth = np.zeros((self.L, self.W), dtype=np.float32)
         self.qx = np.zeros((self.L, self.W), dtype=np.float32)
@@ -358,7 +440,10 @@ class init_tools(abc.ABC):
         self.uy = np.zeros((self.L, self.W), dtype=np.float32)
         self.uw = np.zeros((self.L, self.W), dtype=np.float32)
         self.qs = np.zeros((self.L, self.W), dtype=np.float32)
-
+        self.sand_frac = np.zeros((self.L, self.W), dtype=np.float32) + \
+            self.sand_frac_bc
+        self.active_layer = np.zeros((self.L, self.W), dtype=np.float32) + \
+            self.sand_frac_bc
         self.Vp_dep_sand = np.zeros((self.L, self.W), dtype=np.float32)
         self.Vp_dep_mud = np.zeros((self.L, self.W), dtype=np.float32)
 
@@ -370,7 +455,7 @@ class init_tools(abc.ABC):
         self.sfc_sum = np.zeros_like(self.depth)
 
         # ---- domain ----
-        cell_land = 2
+        cell_land = -2
         cell_channel = 1
         cell_ocean = 0
         cell_edge = -1
@@ -385,7 +470,7 @@ class init_tools(abc.ABC):
                          self._dx * self._S0)
         self.stage[self.cell_type == cell_ocean] = 0.
 
-        self.depth[self.cell_type == cell_ocean] = self.h0
+        self.depth[self.cell_type == cell_ocean] = self.hb
         self.depth[self.cell_type == cell_channel] = self.h0
 
         self.qx[self.cell_type == cell_channel] = self.qw0
@@ -412,7 +497,7 @@ class init_tools(abc.ABC):
 
         self.cell_type[bounds >= min(self.L - 5, self.W / 2 - 5)] = cell_edge
 
-        self.cell_type[:self.L0, :] = -cell_land
+        self.cell_type[:self.L0, :] = cell_land
         self.cell_type[:self.L0, channel_inds:y_channel_max] = cell_channel
 
         self.inlet = np.array(np.unique(np.where(self.cell_type == 1)[1]))
@@ -423,13 +508,19 @@ class init_tools(abc.ABC):
 
         These are preinitialized because the "boxing" for jitted functions is
         expensive, so we avoid boxing up the constants on each iteration.
+
+        .. important::
+
+            If you change any model boundary conditions, you likely should
+            reinitialize the sediment routers (i.e., rerun this method)
         """
         _msg = 'Initializing sediment routers'
         self.log_info(_msg, verbosity=1)
 
         # initialize the MudRouter object
         self._mr = sed_tools.MudRouter(self._dt, self._dx, self.Vp_sed,
-                                       self.u_max, self.U_dep_mud, self.U_ero_mud,
+                                       self.u_max, self.U_dep_mud,
+                                       self.U_ero_mud,
                                        self.ivec_flat, self.jvec_flat,
                                        self.iwalk_flat, self.jwalk_flat,
                                        self.distances_flat, self.dry_depth,
@@ -437,7 +528,8 @@ class init_tools(abc.ABC):
                                        self.theta_mud)
         # initialize the SandRouter object
         self._sr = sed_tools.SandRouter(self._dt, self._dx, self.Vp_sed,
-                                        self.u_max, self.qs0, self._u0, self.U_ero_sand,
+                                        self.u_max, self.qs0, self._u0,
+                                        self.U_ero_sand,
                                         self._f_bedload,
                                         self.ivec_flat, self.jvec_flat,
                                         self.iwalk_flat, self.jwalk_flat,
@@ -445,37 +537,26 @@ class init_tools(abc.ABC):
                                         self._beta, self.stepmax,
                                         self.theta_sand)
 
-    def init_stratigraphy(self):
-        """Creates sparse array to store stratigraphy data."""
-        _msg = 'Initializing stratigraphy storage'
-        self.log_info(_msg, verbosity=1)
-        if self.save_strata:
-
-            self.strata_counter = 0
-
-            self.n_steps = int(max(1, 5 * int(self._save_dt / self.dt)))
-
-            self.strata_sand_frac = lil_matrix((self.L * self.W, self.n_steps),
-                                               dtype=np.float32)
-
-            self.init_eta = self.eta.copy()
-            self.strata_eta = lil_matrix((self.L * self.W, self.n_steps),
-                                         dtype=np.float32)
-
     def init_output_file(self):
         """Creates a netCDF file to store output grids.
 
         Fills with default variables.
 
-        .. warning:: Overwrites an existing netcdf file with the same name.
+        .. warning::
+
+            Overwrites an existing netcdf file with the same name if
+            :attr:`~pyDeltaRCM.model.DeltaModel.clobber_netcdf` is `True`.
 
         """
         _msg = 'Initializing output NetCDF4 file'
         self.log_info(_msg, verbosity=1)
 
+        # set standard/default metadata values in the dict structure
+        if self._save_metadata:
+            self.init_metadata_list()
+
         if (self._save_metadata or
-                self._save_any_grids or
-                self.save_strata):
+                self._save_any_grids):
 
             directory = self.prefix
             filename = 'pyDeltaRCM_output.nc'
@@ -485,7 +566,12 @@ class init_tools(abc.ABC):
                 file=file_path)
             self.log_info(_msg, verbosity=2)
 
-            if os.path.exists(file_path):
+            if (os.path.exists(file_path)) and \
+                    (self._clobber_netcdf is False):
+                raise FileExistsError(
+                    'Existing NetCDF4 output file in target output location.')
+            elif (os.path.exists(file_path)) and \
+                    (self._clobber_netcdf is True):
                 _msg = 'Replacing existing netCDF file'
                 self.logger.warning(_msg)
                 warnings.warn(UserWarning(_msg))
@@ -518,44 +604,18 @@ class init_tools(abc.ABC):
             y[:] = self.y
 
             # set up variables for output data grids
-            if self.save_eta_grids:
-                eta = self.output_netcdf.createVariable(
-                    'eta', 'f4', ('total_time', 'length', 'width'))
-                eta.units = 'meters'
-            if self.save_stage_grids:
-                stage = self.output_netcdf.createVariable(
-                    'stage', 'f4', ('total_time', 'length', 'width'))
-                stage.units = 'meters'
-            if self.save_depth_grids:
-                depth = self.output_netcdf.createVariable(
-                    'depth', 'f4', ('total_time', 'length', 'width'))
-                depth.units = 'meters'
-            if self.save_discharge_grids:
-                discharge = self.output_netcdf.createVariable(
-                    'discharge', 'f4', ('total_time', 'length', 'width'))
-                discharge.units = 'cubic meters per second'
-            if self.save_velocity_grids:
-                velocity = self.output_netcdf.createVariable(
-                    'velocity', 'f4', ('total_time', 'length', 'width'))
-                velocity.units = 'meters per second'
-            if self.save_sedflux_grids:
-                sedflux = self.output_netcdf.createVariable(
-                    'sedflux', 'f4', ('total_time', 'length', 'width'))
-                sedflux.units = 'cubic meters per second'
-            if self.save_discharge_components:
-                discharge_x = self.output_netcdf.createVariable(
-                    'discharge_x', 'f4', ('total_time', 'length', 'width'))
-                discharge_x.units = 'cubic meters per second'
-                discharge_y = self.output_netcdf.createVariable(
-                    'discharge_y', 'f4', ('total_time', 'length', 'width'))
-                discharge_y.units = 'cubic meters per second'
-            if self.save_velocity_components:
-                velocity_x = self.output_netcdf.createVariable(
-                    'velocity_x', 'f4', ('total_time', 'length', 'width'))
-                velocity_x.units = 'meters per second'
-                velocity_y = self.output_netcdf.createVariable(
-                    'velocity_y', 'f4', ('total_time', 'length', 'width'))
-                velocity_y.units = 'meters per second'
+            def _create_grid_variable(varname, varunits,
+                                      vartype='f4', vardims=()):
+                _v = self.output_netcdf.createVariable(
+                    varname, vartype, vardims)
+                _v.units = varunits
+
+            _var_list = list(self._save_var_list.keys())
+            _var_list.remove('meta')
+            for _val in _var_list:
+                _create_grid_variable(_val, self._save_var_list[_val][1],
+                                      self._save_var_list[_val][2],
+                                      self._save_var_list[_val][3])
 
             # set up metadata group and populate variables
             def _create_meta_variable(varname, varvalue, varunits,
@@ -566,32 +626,21 @@ class init_tools(abc.ABC):
                 _v[:] = varvalue
 
             self.output_netcdf.createGroup('meta')
-            # fixed metadata
-            _create_meta_variable('L0', self.L0, 'cells', vartype='i8')
-            _create_meta_variable('N0', self.N0, 'cells', vartype='i8')
-            _create_meta_variable('CTR', self.CTR, 'cells', vartype='i8')
-            _create_meta_variable('dx', self.dx, 'meters')
-            _create_meta_variable('h0', self.h0, 'meters')
-            _create_meta_variable('cell_type', self.cell_type, 'type',
-                                  vartype='i8', vardims=('length', 'width'))
-            # subsidence metadata
-            if self._toggle_subsidence:
-                _create_meta_variable('start_subsidence',
-                                      self.start_subsidence, 'seconds',
-                                      vartype='i8')
-                _create_meta_variable('sigma', self.sigma,
-                                      'meters per timestep',
-                                      vardims=('length', 'width'))
-
-            # time-varying metadata
-            _create_meta_variable('H_SL', None, 'meters',
-                                  vardims=('total_time'))
-            _create_meta_variable('f_bedload', None, 'fraction',
-                                  vardims=('total_time'))
-            _create_meta_variable('C0_percent', None, 'percent',
-                                  vardims=('total_time'))
-            _create_meta_variable('u0', None, 'meters per second',
-                                  vardims=('total_time'))
+            for _val in self._save_var_list['meta'].keys():
+                # time-varying initialize w/ None value, fixed use attribute
+                if (self._save_var_list['meta'][_val][0] is None):
+                    _create_meta_variable(
+                        _val, self._save_var_list['meta'][_val][0],
+                        self._save_var_list['meta'][_val][1],
+                        self._save_var_list['meta'][_val][2],
+                        self._save_var_list['meta'][_val][3])
+                else:
+                    _create_meta_variable(
+                        _val, getattr(self,
+                                      self._save_var_list['meta'][_val][0]),
+                        self._save_var_list['meta'][_val][1],
+                        self._save_var_list['meta'][_val][2],
+                        self._save_var_list['meta'][_val][3])
 
             _msg = 'Output netCDF file created'
             self.log_info(_msg, verbosity=2)
@@ -619,16 +668,49 @@ class init_tools(abc.ABC):
 
             self.sigma = self.subsidence_mask * self._subsidence_rate * self.dt
 
+    def init_metadata_list(self):
+        """Populate the list of metadata information.
+
+        Sets up the dictionary object for the standard metadata.
+        """
+        # fixed metadata
+        self._save_var_list['meta']['L0'] = ['L0', 'cells', 'i8', ()]
+        self._save_var_list['meta']['N0'] = ['N0', 'cells', 'i8', ()]
+        self._save_var_list['meta']['CTR'] = ['CTR', 'cells', 'i8', ()]
+        self._save_var_list['meta']['dx'] = ['dx', 'meters', 'f4', ()]
+        self._save_var_list['meta']['h0'] = ['h0', 'meters', 'f4', ()]
+        self._save_var_list['meta']['hb'] = ['hb', 'meters', 'f4', ()]
+        self._save_var_list['meta']['cell_type'] = ['cell_type',
+                                                    'type', 'i8',
+                                                    ('length', 'width')]
+        # subsidence metadata
+        if self._toggle_subsidence:
+            self._save_var_list['meta']['start_subsidence'] = [
+                'start_subsidence', 'seconds', 'i8', ()
+            ]
+            self._save_var_list['meta']['sigma'] = [
+                'sigma', 'meters per timestep', 'f4',
+                ('length', 'width')
+            ]
+        # time-varying metadata
+        self._save_var_list['meta']['H_SL'] = [None, 'meters', 'f4',
+                                               ('total_time')]
+        self._save_var_list['meta']['f_bedload'] = [None, 'fraction',
+                                                    'f4', ('total_time')]
+        self._save_var_list['meta']['C0_percent'] = [None, 'percent',
+                                                     'f4', ('total_time')]
+        self._save_var_list['meta']['u0'] = [None, 'meters per second',
+                                             'f4', ('total_time')]
+
     def load_checkpoint(self, defer_output=False):
         """Load the checkpoint from the .npz file.
 
         Uses the file at the path determined by `self.prefix` and a file named
         `checkpoint.npz`. There are a few pathways for loading, which depend
-        on 1) the status of the :obj:`save_strata` option, 2) the status of
-        additional grid-saving options, 3) the presence of a netCDF output
-        file at the expected output location, and 4) the status of the
-        :obj:`defer_output` parameter passed to this method as an optional
-        argument.
+        on 1) the status of additional grid-saving options, 2) the presence of
+        a netCDF output file at the expected output location, and 3) the status
+        of the :obj:`defer_output` parameter passed to this method as an
+        optional argument.
 
         As a standard user, you should not need to worry about any of these
         pathways or options. However, if you are developing pyDeltaRCM or
@@ -685,27 +767,8 @@ class init_tools(abc.ABC):
         self.qw = checkpoint['qw']
         self.qx = checkpoint['qx']
         self.qy = checkpoint['qy']
-
-        if self.save_strata:
-            # load the stratigraphy info from the file
-            _msg = 'Loading stratigraphy arrays'
-            self.log_info(_msg, verbosity=2)
-
-            self.n_steps = checkpoint['n_steps']
-            self.init_eta = checkpoint['init_eta']
-            self.strata_counter = checkpoint['strata_counter']
-
-            # reconstruct the strata arrays
-            strata_eta_csr = csr_matrix((checkpoint['eta_data'],
-                                         checkpoint['eta_indices'],
-                                         checkpoint['eta_indptr']),
-                                        shape=checkpoint['eta_shape'])
-            self.strata_eta = strata_eta_csr.tolil()
-            strata_sand_csr = csr_matrix((checkpoint['sand_data'],
-                                          checkpoint['sand_indices'],
-                                          checkpoint['sand_indptr']),
-                                         shape=checkpoint['sand_shape'])
-            self.strata_sand_frac = strata_sand_csr.tolil()
+        self.sand_frac = checkpoint['sand_frac']
+        self.active_layer = checkpoint['active_layer']
 
         # load and set random state to continue as if run hadn't stopped
         _msg = 'Loading random state'
@@ -714,7 +777,7 @@ class init_tools(abc.ABC):
         shared_tools.set_random_state(rng_state)
 
         # handle the case with a netcdf file
-        if ((self._save_any_grids or self._save_metadata or self._save_strata)
+        if ((self._save_any_grids or self._save_metadata)
                 and not defer_output):
 
             # check if the file exists already
@@ -729,25 +792,34 @@ class init_tools(abc.ABC):
                 # create a new file
                 # reset output file counters
                 self._save_iter = int(0)
-                # reset strata fields just loaded from checkpoint
-                if self.save_strata:
-                    self.strata_counter = int(0)
+                self.hook_init_output_file()
                 self.init_output_file()
             else:
+                # check if file is open in another process, if so throw error
+                try:
+                    _dataset = Dataset(file_path, 'r+', format='NETCDF')
+                    _dataset.close()  # if this worked then close it
+                except OSError:
+                    raise RuntimeError(
+                        'Could not open the NetCDF file for checkpointing. '
+                        'This could be because the file is corrupt, or open '
+                        'in another interpreter. Be sure to close any '
+                        'connections to the file before proceeding.')
+                # if not open elsewhere, then proceed
                 # rename the old netCDF4 file
                 _msg = 'Renaming old NetCDF4 output file'
                 self.log_info(_msg, verbosity=2)
-                _tmp_name = os.path.join(self.prefix, 'old_pyDeltaRCM_output.nc')
+                _tmp_name = os.path.join(self.prefix,
+                                         'old_pyDeltaRCM_output.nc')
                 os.rename(file_path, _tmp_name)
 
                 # write dims / attributes / variables to new netCDF file
-                # except the things defined by output_strata()
                 _msg = 'Creating NetCDF4 output file'
                 self.log_info(_msg, verbosity=2)
 
-                # list of things to not copy over
-                dimtoignore = ['total_strata_age']
-                vartoignore = ['strata_age', 'strata_sand_frac', 'strata_depth']
+                # populate default metadata list
+                if self._save_metadata:
+                    self.init_metadata_list()
 
                 # copy data from old netCDF4 into new one
                 with Dataset(_tmp_name) as src, Dataset(file_path, 'w',
@@ -757,11 +829,10 @@ class init_tools(abc.ABC):
                         dst.setncattr(name, src.getncattr(name))
                     # copy dimensions
                     for name, dimension in src.dimensions.items():
-                        if name not in dimtoignore:
-                            if dimension.isunlimited():
-                                dst.createDimension(name, None)
-                            else:
-                                dst.createDimension(name, len(dimension))
+                        if dimension.isunlimited():
+                            dst.createDimension(name, None)
+                        else:
+                            dst.createDimension(name, len(dimension))
                     # copy groups (meta)
                     for name in src.groups.keys():
                         dst.createGroup(name)
@@ -773,10 +844,9 @@ class init_tools(abc.ABC):
                                 src.groups[name].variables[vname][:]
                     # copy variables except ones to exclude
                     for name, variable in src.variables.items():
-                        if name not in vartoignore:
-                            dst.createVariable(name, variable.datatype,
-                                               variable.dimensions)
-                            dst.variables[name][:] = src.variables[name][:]
+                        dst.createVariable(name, variable.datatype,
+                                           variable.dimensions)
+                        dst.variables[name][:] = src.variables[name][:]
 
                 # set object attribute for model
                 self.output_netcdf = Dataset(file_path, 'r+', format='NETCDF4')
