@@ -10,6 +10,7 @@ from .water_tools import water_tools
 from .init_tools import init_tools
 from .hook_tools import hook_tools
 from .debug_tools import debug_tools
+from .shared_tools import _get_version
 
 
 class DeltaModel(iteration_tools, sed_tools, water_tools,
@@ -71,6 +72,8 @@ class DeltaModel(iteration_tools, sed_tools, water_tools,
         :doc:`/guides/user_guide`.
 
         """
+        self.__pyDeltaRCM_version__ = _get_version()
+
         self._time = 0.
         self._time_iter = int(0)
         self._save_time_since_data = float("inf")  # force save on t==0
@@ -81,6 +84,13 @@ class DeltaModel(iteration_tools, sed_tools, water_tools,
         _src_dir = os.path.realpath(os.path.dirname(__file__))
         self.default_file = os.path.join(_src_dir, 'default.yml')
 
+        # infrastructure for custom yaml parameters if needed
+        if hasattr(self, 'subclass_parameters') is False:
+            self.subclass_parameters = dict()  # init dict for custom params
+
+        # check for any deprecated hooks
+        self._check_deprecated_hooks()
+
         # import the input file
         self.hook_import_files()  # user hook
         self.import_files(kwargs)
@@ -110,10 +120,6 @@ class DeltaModel(iteration_tools, sed_tools, water_tools,
         # set up the subsidence fields
         self.init_subsidence()
 
-        # initialize the stratigraphy infrastructure
-        self.hook_init_stratigraphy()
-        self.init_stratigraphy()
-
         # if resume flag set to True, load checkpoint, open netCDF4
         if self.resume_checkpoint:
             # load values from the checkpoint and don't init final features
@@ -123,6 +129,7 @@ class DeltaModel(iteration_tools, sed_tools, water_tools,
         else:
             # initialize the output file
             if not defer_output:
+                self.hook_init_output_file()
                 self.init_output_file()
 
                 # record initial conditions
@@ -137,21 +144,31 @@ class DeltaModel(iteration_tools, sed_tools, water_tools,
         self.log_model_time()
 
     def update(self):
-        """Run the model for one full instance.
+        """Run the model for one full timestep.
 
         This method handles the input/output from the model, orchestrating the
-        various morphodynamic and basin-scale processes, and incrementing the
-        model time-tracking attributes. This method calls, in sequence:
+        various morphodynamic and basin-scale processes (from sub-methods),
+        and incrementing the model time-tracking attributes. This method
+        calls, in sequence:
 
             * the routine to run one timestep (i.e., water surface estimation
-              and sediment routing, :meth:`run_one_timestep`)
-            * the basin subsidence update pattern (:meth:`apply_subsidence`)
-            * the timestep finalization routine (:meth:`finalize_timestep`)
-            * straigraphy updating routine (:meth:`record_stratigraphy`)
+              and sediment routing;
+              :meth:`~pyDeltaRCM.iteration_tools.iteration_tools.solve_water_and_sediment_timestep`)
+            * the basin subsidence update;
+              (:meth:`~pyDeltaRCM.iteration_tools.iteration_tools.apply_subsidence`)
+            * the timestep finalization routine (applying boundary conditions);
+              (:meth:`~pyDeltaRCM.iteration_tools.iteration_tools.finalize_timestep`)
+            * the internal time-tracking attributes of the model are updated
+            * the routine to output data to the NetCDF file;
+              (:meth:`~pyDeltaRCM.iteration_tools.iteration_tools.output_data`)
+            * the routine to output a checkpoint file;
+              (:meth:`~pyDeltaRCM.iteration_tools.iteration_tools.output_checkpoint`)
 
         If you attempt to override the ``update`` routine, you must implement
         these operations at a minimum. More likely, you can implement what you
-        need to by just overriding one of the methods called by `update`.
+        need to by utilizing a model `hook`, or overriding one of the methods
+        called by `update`. For more information on customizing the model, see
+        the :ref:`complete guide <customize_the_model>`.
 
         Parameters
         ----------
@@ -169,8 +186,8 @@ class DeltaModel(iteration_tools, sed_tools, water_tools,
             raise RuntimeError('Cannot update model, model already finalized!')
 
         # update the model, i.e., the actual model morphodynamics
-        self.hook_run_one_timestep()
-        self.run_one_timestep()
+        self.hook_solve_water_and_sediment_timestep()
+        self.solve_water_and_sediment_timestep()
 
         self.hook_apply_subsidence()
         self.apply_subsidence()
@@ -213,9 +230,6 @@ class DeltaModel(iteration_tools, sed_tools, water_tools,
             raise RuntimeError('Cannot finalize model, '
                                'model already finalized!')
 
-        self.hook_record_final_stratigraphy()
-        self.record_final_stratigraphy()
-
         try:
             self.output_netcdf.close()
             _msg = 'Closed output NetCDF4 file'
@@ -434,6 +448,22 @@ class DeltaModel(iteration_tools, sed_tools, water_tools,
     def h0(self, h0):
         self._h0 = h0
 
+    @property
+    def hb(self):
+        """
+        hb is the basin depth in meters.
+
+        hb is the basin depth in meters. This parameter must be an *integer*
+        or *float*, or *None*. If no value is supplied for this parameter,
+        then the value of h0 is used to determine the basin depth (i.e., they
+        are the same).
+        """
+        return self._hb
+
+    @hb.setter
+    def hb(self, hb):
+        self._hb = hb
+
     @property
     def H_SL(self):
         """
@@ -516,6 +546,29 @@ class DeltaModel(iteration_tools, sed_tools, water_tools,
                              ' inclusive.')
         self._f_bedload = f_bedload
 
+    @property
+    def active_layer_thickness(self):
+        """
+        Active layer thickness for sand fraction.
+
+        The active layer thickness is the depth to which the sand fraction
+        values are maintained in the event of erosion from one timestep to the
+        next. When erosion exceeds the depth of the active layer, the boundary
+        condition is to fill that cell with a value of 0 (maybe this should be
+        -1 so that it is clear which cells are not truly mud but unknown
+        sediment content based on the active layer thickness?)
+        """
+        return self._active_layer_thickness
+
+    @active_layer_thickness.setter
+    def active_layer_thickness(self, active_layer_thickness):
+        if active_layer_thickness is None:
+            active_layer_thickness = self._h0 / 2
+        elif active_layer_thickness < 0:
+            raise ValueError('active_layer thickness must be greater than'
+                             ' or equal to 0, cannot be negative.')
+        self._active_layer_thickness = active_layer_thickness
+
     @property
     def C0_percent(self):
         """
@@ -591,6 +644,12 @@ class DeltaModel(iteration_tools, sed_tools, water_tools,
 
     @save_eta_figs.setter
     def save_eta_figs(self, save_eta_figs):
+        if (save_eta_figs is True) and \
+          ('eta' not in self._save_fig_list.keys()):
+            self._save_fig_list['eta'] = ['eta']
+        elif ((save_eta_figs is False) and
+              ('eta' in self._save_fig_list.keys())):
+            del self._save_fig_list['eta']
         self._save_eta_figs = save_eta_figs
 
     @property
@@ -602,6 +661,12 @@ class DeltaModel(iteration_tools, sed_tools, water_tools,
 
     @save_stage_figs.setter
     def save_stage_figs(self, save_stage_figs):
+        if (save_stage_figs is True) and \
+          ('stage' not in self._save_fig_list.keys()):
+            self._save_fig_list['stage'] = ['stage']
+        elif ((save_stage_figs is False) and
+              ('stage' in self._save_fig_list.keys())):
+            del self._save_fig_list['stage']
         self._save_stage_figs = save_stage_figs
 
     @property
@@ -613,6 +678,12 @@ class DeltaModel(iteration_tools, sed_tools, water_tools,
 
     @save_depth_figs.setter
     def save_depth_figs(self, save_depth_figs):
+        if (save_depth_figs is True) and \
+          ('depth' not in self._save_fig_list.keys()):
+            self._save_fig_list['depth'] = ['depth']
+        elif ((save_depth_figs is False) and
+              ('depth' in self._save_fig_list.keys())):
+            del self._save_fig_list['depth']
         self._save_depth_figs = save_depth_figs
 
     @property
@@ -624,6 +695,12 @@ class DeltaModel(iteration_tools, sed_tools, water_tools,
 
     @save_discharge_figs.setter
     def save_discharge_figs(self, save_discharge_figs):
+        if (save_discharge_figs is True) and \
+          ('discharge' not in self._save_fig_list.keys()):
+            self._save_fig_list['discharge'] = ['qw']
+        elif ((save_discharge_figs is False) and
+              ('discharge' in self._save_fig_list)):
+            del self._save_fig_list['discharge']
         self._save_discharge_figs = save_discharge_figs
 
     @property
@@ -635,6 +712,12 @@ class DeltaModel(iteration_tools, sed_tools, water_tools,
 
     @save_velocity_figs.setter
     def save_velocity_figs(self, save_velocity_figs):
+        if (save_velocity_figs is True) and \
+          ('velocity' not in self._save_fig_list.keys()):
+            self._save_fig_list['velocity'] = ['uw']
+        elif ((save_velocity_figs is False) and
+              ('velocity' in self._save_fig_list.keys())):
+            del self._save_fig_list['sedflux']
         self._save_velocity_figs = save_velocity_figs
 
     @property
@@ -646,8 +729,32 @@ class DeltaModel(iteration_tools, sed_tools, water_tools,
 
     @save_sedflux_figs.setter
     def save_sedflux_figs(self, save_sedflux_figs):
+        if (save_sedflux_figs is True) and \
+          ('sedflux' not in self._save_fig_list.keys()):
+            self._save_fig_list['sedflux'] = ['qs']
+        elif ((save_sedflux_figs is False) and
+              ('sedflux' in self._save_fig_list)):
+            del self._save_fig_list['sedflux']
         self._save_sedflux_figs = save_sedflux_figs
 
+    @property
+    def save_sandfrac_figs(self):
+        """
+        save_sandfrac_figs controls whether or not figures of the sediment
+        surface (i.e., the bed) sand fraction are saved.
+        """
+        return self._save_sandfrac_figs
+
+    @save_sandfrac_figs.setter
+    def save_sandfrac_figs(self, save_sandfrac_figs):
+        if (save_sandfrac_figs is True) and \
+          ('sandfrac' not in self._save_fig_list.keys()):
+            self._save_fig_list['sandfrac'] = ['sand_frac']
+        elif ((save_sandfrac_figs is False) and
+              ('sandfrac' in self._save_fig_list)):
+            del self._save_fig_list['sandfrac']
+        self._save_sandfrac_figs = save_sandfrac_figs
+
     @property
     def save_figs_sequential(self):
         """
@@ -697,6 +804,13 @@ class DeltaModel(iteration_tools, sed_tools, water_tools,
 
     @save_eta_grids.setter
     def save_eta_grids(self, save_eta_grids):
+        if (save_eta_grids is True) and \
+          ('eta' not in self._save_var_list.keys()):
+            self._save_var_list['eta'] = ['eta', 'meters', 'f4',
+                                          ('total_time', 'length', 'width')]
+        elif ((save_eta_grids is False) and
+              ('eta' in self._save_var_list.keys())):
+            del self._save_var_list['eta']
         self._save_eta_grids = save_eta_grids
 
     @property
@@ -708,6 +822,13 @@ class DeltaModel(iteration_tools, sed_tools, water_tools,
 
     @save_stage_grids.setter
     def save_stage_grids(self, save_stage_grids):
+        if (save_stage_grids is True) and \
+          ('stage' not in self._save_var_list.keys()):
+            self._save_var_list['stage'] = ['stage', 'meters', 'f4',
+                                            ('total_time', 'length', 'width')]
+        elif ((save_stage_grids is False) and
+              ('stage' in self._save_var_list.keys())):
+            del self._save_var_list['stage']
         self._save_stage_grids = save_stage_grids
 
     @property
@@ -719,6 +840,13 @@ class DeltaModel(iteration_tools, sed_tools, water_tools,
 
     @save_depth_grids.setter
     def save_depth_grids(self, save_depth_grids):
+        if (save_depth_grids is True) and \
+          ('depth' not in self._save_var_list.keys()):
+            self._save_var_list['depth'] = ['depth', 'meters', 'f4',
+                                            ('total_time', 'length', 'width')]
+        elif ((save_depth_grids is False) and
+              ('depth' in self._save_var_list.keys())):
+            del self._save_var_list['depth']
         self._save_depth_grids = save_depth_grids
 
     @property
@@ -730,6 +858,16 @@ class DeltaModel(iteration_tools, sed_tools, water_tools,
 
     @save_discharge_grids.setter
     def save_discharge_grids(self, save_discharge_grids):
+        if (save_discharge_grids is True) and \
+          ('discharge' not in self._save_var_list.keys()):
+            self._save_var_list['discharge'] = ['qw',
+                                                'cubic meters per second',
+                                                'f4',
+                                                ('total_time', 'length',
+                                                 'width')]
+        elif ((save_discharge_grids is False) and
+              ('discharge' in self._save_var_list.keys())):
+            del self._save_var_list['discharge']
         self._save_discharge_grids = save_discharge_grids
 
     @property
@@ -741,6 +879,14 @@ class DeltaModel(iteration_tools, sed_tools, water_tools,
 
     @save_velocity_grids.setter
     def save_velocity_grids(self, save_velocity_grids):
+        if (save_velocity_grids is True) and \
+          ('velocity' not in self._save_var_list.keys()):
+            self._save_var_list['velocity'] = ['uw', 'meters per second', 'f4',
+                                               ('total_time', 'length',
+                                                'width')]
+        elif ((save_velocity_grids is False) and
+              ('velocity' in self._save_var_list.keys())):
+            del self._save_var_list['velocity']
         self._save_velocity_grids = save_velocity_grids
 
     @property
@@ -752,8 +898,37 @@ class DeltaModel(iteration_tools, sed_tools, water_tools,
 
     @save_sedflux_grids.setter
     def save_sedflux_grids(self, save_sedflux_grids):
+        if (save_sedflux_grids is True) and \
+          ('sedflux' not in self._save_var_list.keys()):
+            self._save_var_list['sedflux'] = ['qs', 'cubic meters per second',
+                                              'f4',
+                                              ('total_time', 'length',
+                                               'width')]
+        elif ((save_sedflux_grids is False) and
+              ('sedflux' in self._save_var_list.keys())):
+            del self._save_var_list['sedflux']
         self._save_sedflux_grids = save_sedflux_grids
 
+    @property
+    def save_sandfrac_grids(self):
+        """
+        save_sandfrac_grids controls whether or not the sediment
+        surface (i.e., the bed) sand fraction is saved.
+        """
+        return self._save_sandfrac_grids
+
+    @save_sandfrac_grids.setter
+    def save_sandfrac_grids(self, save_sandfrac_grids):
+        if (save_sandfrac_grids is True) and \
+          ('sandfrac' not in self._save_var_list.keys()):
+            self._save_var_list['sandfrac'] = ['sand_frac', 'fraction', 'f4',
+                                               ('total_time', 'length',
+                                                'width')]
+        elif ((save_sandfrac_grids is False) and
+              ('sandfrac' in self._save_var_list.keys())):
+            del self._save_var_list['sandfrac']
+        self._save_sandfrac_grids = save_sandfrac_grids
+
     @property
     def save_discharge_components(self):
         """
@@ -763,6 +938,20 @@ class DeltaModel(iteration_tools, sed_tools, water_tools,
 
     @save_discharge_components.setter
     def save_discharge_components(self, save_discharge_components):
+        if (save_discharge_components is True):
+            if ('discharge_x' not in self._save_var_list.keys()):
+                self._save_var_list['discharge_x'] = [
+                    'qx', 'cubic meters per second', 'f4',
+                    ('total_time', 'length', 'width')]
+            if ('discharge_y' not in self._save_var_list.keys()):
+                self._save_var_list['discharge_y'] = [
+                    'qy', 'cubic meters per second', 'f4',
+                    ('total_time', 'length', 'width')]
+        elif (save_discharge_components is False):
+            if ('discharge_x' in self._save_var_list.keys()):
+                del self._save_var_list['discharge_x']
+            if ('discharge_y' in self._save_var_list.keys()):
+                del self._save_var_list['discharge_y']
         self._save_discharge_components = save_discharge_components
 
     @property
@@ -774,6 +963,20 @@ class DeltaModel(iteration_tools, sed_tools, water_tools,
 
     @save_velocity_components.setter
     def save_velocity_components(self, save_velocity_components):
+        if (save_velocity_components is True):
+            if ('velocity_x' not in self._save_var_list.keys()):
+                self._save_var_list['velocity_x'] = [
+                    'ux', 'meters per second', 'f4',
+                    ('total_time', 'length', 'width')]
+            if ('velocity_y' not in self._save_var_list.keys()):
+                self._save_var_list['velocity_y'] = [
+                    'uy', 'meters per second', 'f4',
+                    ('total_time', 'length', 'width')]
+        elif (save_velocity_components is False):
+            if ('velocity_x' in self._save_var_list.keys()):
+                del self._save_var_list['velocity_x']
+            if ('velocity_y' in self._save_var_list.keys()):
+                del self._save_var_list['velocity_y']
         self._save_velocity_components = save_velocity_components
 
     @property
@@ -796,19 +999,10 @@ class DeltaModel(iteration_tools, sed_tools, water_tools,
 
     @checkpoint_dt.setter
     def checkpoint_dt(self, checkpoint_dt):
+        if checkpoint_dt is None:
+            checkpoint_dt = self._save_dt
         self._checkpoint_dt = checkpoint_dt
 
-    @property
-    def save_strata(self):
-        """
-        save_strata controls whether or not stratigraphy information is saved.
-        """
-        return self._save_strata
-
-    @save_strata.setter
-    def save_strata(self, save_strata):
-        self._save_strata = save_strata
-
     @property
     def save_checkpoint(self):
         """
@@ -1049,6 +1243,23 @@ class DeltaModel(iteration_tools, sed_tools, water_tools,
     def stepmax(self, stepmax):
         self._stepmax = stepmax
 
+    @property
+    def clobber_netcdf(self):
+        """
+        Allows overwriting (clobbering) of an existing netCDF output file.
+
+        Default behavior, clobber_netcdf: False, is for the model to raise an
+        error during initialization if a netCDF output file is found in the
+        location specified by :attr:`out_dir`. If the clobber_netcdf parameter
+        is instead set to True, then if there is an existing netCDF output file
+        in the target folder, it will be "clobbered" or overwritten.
+        """
+        return self._clobber_netcdf
+
+    @clobber_netcdf.setter
+    def clobber_netcdf(self, clobber_netcdf):
+        self._clobber_netcdf = clobber_netcdf
+
     @property
     def time(self):
         """Elapsed model time in seconds.
@@ -1138,7 +1349,7 @@ class DeltaModel(iteration_tools, sed_tools, water_tools,
     @channel_flow_velocity.setter
     def channel_flow_velocity(self, new_u0):
         self.u0 = new_u0
-        self.create_other_variables()
+        self.create_boundary_conditions()
         self.init_sediment_routers()
 
     @property
@@ -1149,7 +1360,7 @@ class DeltaModel(iteration_tools, sed_tools, water_tools,
     @channel_width.setter
     def channel_width(self, new_N0_meters):
         self.N0_meters = new_N0_meters
-        self.create_other_variables()
+        self.create_boundary_conditions()
         self.init_sediment_routers()
         if self.channel_width != new_N0_meters:
             warnings.warn(UserWarning(
@@ -1165,7 +1376,7 @@ class DeltaModel(iteration_tools, sed_tools, water_tools,
     @channel_flow_depth.setter
     def channel_flow_depth(self, new_d):
         self.h0 = new_d
-        self.create_other_variables()
+        self.create_boundary_conditions()
         self.init_sediment_routers()
 
     @property
@@ -1203,7 +1414,7 @@ class DeltaModel(iteration_tools, sed_tools, water_tools,
     @influx_sediment_concentration.setter
     def influx_sediment_concentration(self, new_C0):
         self.C0_percent = new_C0 * 100
-        self.create_other_variables()
+        self.create_boundary_conditions()
         self.init_sediment_routers()
 
     @property
@@ -1220,3 +1431,12 @@ class DeltaModel(iteration_tools, sed_tools, water_tools,
     def bed_elevation(self):
         """Get bed elevation."""
         return self.eta
+
+    @property
+    def sand_frac_bc(self):
+        """Sand fraction boundary condition."""
+        return self._sand_frac_bc
+
+    @sand_frac_bc.setter
+    def sand_frac_bc(self, sand_frac_bc):
+        self._sand_frac_bc = sand_frac_bc
