@@ -5,6 +5,7 @@ import unittest.mock as mock
 
 import numpy as np
 import os
+from netCDF4 import Dataset
 
 from pyDeltaRCM.model import DeltaModel
 
@@ -149,6 +150,30 @@ class TestModelDomainSetup:
         assert np.any(delta.sfc_sum) == 0
 
 
+class TestCreateBoundaryConditions:
+
+    # base case during init is covered by tests elsewhere!
+
+    def test_change_variable_updated_bcs(self, tmp_path):
+        p = utilities.yaml_from_dict(tmp_path, 'input.yaml')
+        _delta = DeltaModel(input_file=p)
+
+        # what is Qw0 before?
+        assert _delta.Qw0 == 1250.0
+
+        # change u0
+        _delta.u0 = 2
+
+        # nothing has happened to Qw0 yet
+        assert _delta.u0 == 2
+        assert _delta.Qw0 == 1250.0
+
+        # now call to recreate, see what happened
+        _delta.create_boundary_conditions()
+        assert _delta.u0 == 2
+        assert _delta.Qw0 == 2500.0
+
+
 class TestInitSubsidence:
 
     def test_subsidence_bounds(self, tmp_path):
@@ -174,57 +199,12 @@ class TestInitSubsidence:
 class TestLoadCheckpoint:
 
     @mock.patch('pyDeltaRCM.shared_tools.set_random_state')
-    def test_load_standard_strata(self, patched, tmp_path):
-        """Test that a run can be resumed when there are outputs.
-        """
-        # create one delta, just to have a checkpoint file
-        p = utilities.yaml_from_dict(tmp_path, 'input.yaml',
-                                     {'save_checkpoint': True})
-        _delta = DeltaModel(input_file=p)
-
-        # make mocks
-        _delta.log_info = mock.MagicMock()
-        _delta.logger = mock.MagicMock()
-        _delta.init_output_file = mock.MagicMock()
-
-        # close the file so can be safely opened in load
-        _delta.output_netcdf.close()
-
-        # check checkpoint exists
-        assert os.path.isfile(os.path.join(
-            _delta.prefix, 'checkpoint.npz'))
-        assert os.path.isfile(os.path.join(
-            _delta.prefix, 'pyDeltaRCM_output.nc'))
-        assert hasattr(_delta, 'strata_counter')
-
-        # now mess up a field
-        _eta0 = np.copy(_delta.eta)
-        _rand_field = np.random.uniform(0, 1, size=_delta.eta.shape)
-        _delta.eta = _rand_field
-        assert np.all(_delta.eta == _rand_field)
-
-        # now resume from the checkpoint to restore the field
-        _delta.load_checkpoint()
-
-        # check that fields match
-        assert np.all(_delta.eta == _eta0)
-        assert hasattr(_delta, 'strata_counter')
-
-        # assertions on function calls
-        _strat_call = [mock.call('Loading stratigraphy arrays', verbosity=2)]
-        _delta.log_info.assert_has_calls(_strat_call, any_order=True)
-        _delta.logger.assert_not_called()
-        _delta.init_output_file.assert_not_called()
-        patched.assert_called()
-
-    @mock.patch('pyDeltaRCM.shared_tools.set_random_state')
-    def test_load_standard_grid_nostrata(self, patched, tmp_path):
+    def test_load_standard_grid(self, patched, tmp_path):
         """Test that a run can be resumed when there are outputs.
         """
         # create one delta, just to have a checkpoint file
         p = utilities.yaml_from_dict(tmp_path, 'input.yaml',
                                      {'save_checkpoint': True,
-                                      'save_strata': False,
                                       'save_eta_grids': True})
         _delta = DeltaModel(input_file=p)
 
@@ -241,7 +221,6 @@ class TestLoadCheckpoint:
             _delta.prefix, 'checkpoint.npz'))
         assert os.path.isfile(os.path.join(
             _delta.prefix, 'pyDeltaRCM_output.nc'))
-        assert not hasattr(_delta, 'strata_counter')
 
         # now mess up a field
         _eta0 = np.copy(_delta.eta)
@@ -254,7 +233,6 @@ class TestLoadCheckpoint:
 
         # check that fields match
         assert np.all(_delta.eta == _eta0)
-        assert not hasattr(_delta, 'strata_counter')
 
         # assertions on function calls
         _call = [mock.call('Renaming old NetCDF4 output file', verbosity=2)]
@@ -271,8 +249,7 @@ class TestLoadCheckpoint:
         """
         # create one delta, just to have a checkpoint file
         p = utilities.yaml_from_dict(tmp_path, 'input.yaml',
-                                     {'save_checkpoint': True,
-                                      'save_strata': False})
+                                     {'save_checkpoint': True})
         _delta = DeltaModel(input_file=p)
 
         # make mocks
@@ -284,7 +261,6 @@ class TestLoadCheckpoint:
             _delta.prefix, 'checkpoint.npz'))
         assert not os.path.isfile(os.path.join(
             _delta.prefix, 'pyDeltaRCM_output.nc'))
-        assert not hasattr(_delta, 'strata_counter')
 
         # now mess up a field
         _eta0 = np.copy(_delta.eta)
@@ -297,7 +273,6 @@ class TestLoadCheckpoint:
 
         # check that fields match
         assert np.all(_delta.eta == _eta0)
-        assert not hasattr(_delta, 'strata_counter')
 
         # assertions on function calls
         _delta.log_info.assert_called()
@@ -315,7 +290,6 @@ class TestLoadCheckpoint:
         # define a yaml with NO outputs, but checkpoint
         p = utilities.yaml_from_dict(tmp_path, 'input.yaml',
                                      {'save_checkpoint': True,
-                                      'save_strata': False,
                                       'save_eta_grids': True})
         _delta = DeltaModel(input_file=p)
 
@@ -347,7 +321,6 @@ class TestLoadCheckpoint:
 
         # check that fields match
         assert np.all(_delta.eta == _eta0)
-        assert not hasattr(_delta, 'strata_counter')
         assert _delta._save_iter == 0
 
         # assertions on function calls
@@ -356,6 +329,39 @@ class TestLoadCheckpoint:
         _delta.init_output_file.assert_called()
         patched.assert_called()
 
+    @mock.patch('pyDeltaRCM.shared_tools.set_random_state')
+    def test_load_already_open_netcdf_error(self, patched, tmp_path):
+        """
+        Test that a checkpoint can be loaded when the load expects there to be
+        a netcdf file. This will create a new netcdf file and raise a
+        warning.
+        """
+        # define a yaml with an output, and checkpoint
+        p = utilities.yaml_from_dict(tmp_path, 'input.yaml',
+                                     {'save_checkpoint': True,
+                                      'save_eta_grids': True})
+        _delta = DeltaModel(input_file=p)
+
+        # make mocks
+        _delta.log_info = mock.MagicMock()
+        _delta.logger = mock.MagicMock()
+        _delta.init_output_file = mock.MagicMock()
+
+        # close the file so can be safely opened in load
+        _delta.output_netcdf.close()
+
+        # check that files exist, and then open the nc back up
+        assert os.path.isfile(os.path.join(
+            _delta.prefix, 'pyDeltaRCM_output.nc'))
+        assert os.path.isfile(os.path.join(
+            _delta.prefix, 'checkpoint.npz'))
+        _ = Dataset(os.path.join(
+            _delta.prefix, 'pyDeltaRCM_output.nc'))
+
+        # now try to resume a model and should throw error
+        with pytest.raises(RuntimeError):
+            _delta.load_checkpoint()
+
 
 class TestSettingConstants:
     """
@@ -455,6 +461,37 @@ class TestSettingParametersFromYAMLFile:
         _delta = DeltaModel(input_file=p)
         assert _delta.itermax == 6
 
+    def test_h0(self, tmp_path):
+        p = utilities.yaml_from_dict(tmp_path, 'input.yaml',
+                                     {'h0': 7.5})
+        _delta = DeltaModel(input_file=p)
+        assert _delta.h0 == 7.5
+
+        p = utilities.yaml_from_dict(tmp_path, 'input.yaml',
+                                     {'h0': int(7)})
+        _delta = DeltaModel(input_file=p)
+        assert _delta.h0 == 7
+
+    def test_hb(self, tmp_path):
+        # take default from h0 if not given:
+        p = utilities.yaml_from_dict(tmp_path, 'input.yaml',
+                                     {'h0': 7.5})
+        _delta = DeltaModel(input_file=p)
+        assert _delta.h0 == 7.5
+        assert _delta.hb == 7.5
+
+        p = utilities.yaml_from_dict(tmp_path, 'input.yaml',
+                                     {'hb': 7.5})
+        _delta = DeltaModel(input_file=p)
+        assert _delta.h0 == 5
+        assert _delta.hb == 7.5
+
+        p = utilities.yaml_from_dict(tmp_path, 'input.yaml',
+                                     {'hb': int(7)})
+        _delta = DeltaModel(input_file=p)
+        assert _delta.h0 == 5
+        assert _delta.hb == 7
+
     def test_Nsmooth(self, tmp_path):
         p = utilities.yaml_from_dict(tmp_path, 'input.yaml', {'Nsmooth': 6})
         _delta = DeltaModel(input_file=p)
@@ -506,92 +543,131 @@ class TestSettingParametersFromYAMLFile:
                                      {'save_eta_grids': True})
         _delta = DeltaModel(input_file=p)
         assert _delta._save_any_grids is True
-        assert _delta._save_any_figs is False
+        assert len(_delta._save_fig_list) == 0
+        assert 'eta' in _delta._save_var_list.keys()
 
     def test_save_depth_grids(self, tmp_path):
         p = utilities.yaml_from_dict(tmp_path, 'input.yaml',
                                      {'save_depth_grids': True})
         _delta = DeltaModel(input_file=p)
         assert _delta._save_any_grids is True
-        assert _delta._save_any_figs is False
+        assert len(_delta._save_fig_list) == 0
+        assert 'depth' in _delta._save_var_list.keys()
 
     def test_save_stage_grids(self, tmp_path):
         p = utilities.yaml_from_dict(tmp_path, 'input.yaml',
                                      {'save_stage_grids': True})
         _delta = DeltaModel(input_file=p)
         assert _delta._save_any_grids is True
-        assert _delta._save_any_figs is False
+        assert len(_delta._save_fig_list) == 0
+        assert 'stage' in _delta._save_var_list.keys()
 
     def test_save_discharge_grids(self, tmp_path):
         p = utilities.yaml_from_dict(tmp_path, 'input.yaml',
                                      {'save_discharge_grids': True})
         _delta = DeltaModel(input_file=p)
         assert _delta._save_any_grids is True
-        assert _delta._save_any_figs is False
+        assert len(_delta._save_fig_list) == 0
+        assert 'discharge' in _delta._save_var_list.keys()
 
     def test_save_velocity_grids(self, tmp_path):
         p = utilities.yaml_from_dict(tmp_path, 'input.yaml',
                                      {'save_velocity_grids': True})
         _delta = DeltaModel(input_file=p)
         assert _delta._save_any_grids is True
-        assert _delta._save_any_figs is False
+        assert len(_delta._save_fig_list) == 0
+        assert 'velocity' in _delta._save_var_list.keys()
+
+    def test_save_sedflux_grids(self, tmp_path):
+        p = utilities.yaml_from_dict(tmp_path, 'input.yaml',
+                                     {'save_sedflux_grids': True})
+        _delta = DeltaModel(input_file=p)
+        assert _delta._save_any_grids is True
+        assert len(_delta._save_fig_list) == 0
+        assert 'sedflux' in _delta._save_var_list.keys()
+
+    def test_save_sandfrac_grids(self, tmp_path):
+        p = utilities.yaml_from_dict(tmp_path, 'input.yaml',
+                                     {'save_sandfrac_grids': True})
+        _delta = DeltaModel(input_file=p)
+        assert _delta._save_any_grids is True
+        assert len(_delta._save_fig_list) == 0
+        assert 'sandfrac' in _delta._save_var_list.keys()
 
     def test_save_discharge_components(self, tmp_path):
         p = utilities.yaml_from_dict(tmp_path, 'input.yaml',
                                      {'save_discharge_components': True})
         _delta = DeltaModel(input_file=p)
         assert _delta._save_any_grids is True
-        assert _delta._save_any_figs is False
+        assert len(_delta._save_fig_list) == 0
         assert _delta._save_discharge_components is True
+        assert 'discharge_x' in _delta._save_var_list.keys()
+        assert 'discharge_y' in _delta._save_var_list.keys()
 
     def test_save_velocity_components(self, tmp_path):
         p = utilities.yaml_from_dict(tmp_path, 'input.yaml',
                                      {'save_velocity_components': True})
         _delta = DeltaModel(input_file=p)
         assert _delta._save_any_grids is True
-        assert _delta._save_any_figs is False
+        assert len(_delta._save_fig_list) == 0
         assert _delta._save_velocity_components is True
+        assert 'velocity_x' in _delta._save_var_list.keys()
+        assert 'velocity_y' in _delta._save_var_list.keys()
 
     def test_save_eta_figs(self, tmp_path):
         p = utilities.yaml_from_dict(tmp_path, 'input.yaml',
                                      {'save_eta_figs': True})
         _delta = DeltaModel(input_file=p)
-        assert _delta._save_any_figs is True
+        assert len(_delta._save_fig_list) > 0
         assert _delta._save_any_grids is False
 
     def test_save_depth_figs(self, tmp_path):
         p = utilities.yaml_from_dict(tmp_path, 'input.yaml',
                                      {'save_depth_figs': True})
         _delta = DeltaModel(input_file=p)
-        assert _delta._save_any_figs is True
+        assert len(_delta._save_fig_list) > 0
         assert _delta._save_any_grids is False
 
     def test_save_stage_figs(self, tmp_path):
         p = utilities.yaml_from_dict(tmp_path, 'input.yaml',
                                      {'save_stage_figs': True})
         _delta = DeltaModel(input_file=p)
-        assert _delta._save_any_figs is True
+        assert len(_delta._save_fig_list) > 0
         assert _delta._save_any_grids is False
 
     def test_save_discharge_figs(self, tmp_path):
         p = utilities.yaml_from_dict(tmp_path, 'input.yaml',
                                      {'save_discharge_figs': True})
         _delta = DeltaModel(input_file=p)
-        assert _delta._save_any_figs is True
+        assert len(_delta._save_fig_list) > 0
         assert _delta._save_any_grids is False
 
     def test_save_velocity_figs(self, tmp_path):
         p = utilities.yaml_from_dict(tmp_path, 'input.yaml',
                                      {'save_velocity_figs': True})
         _delta = DeltaModel(input_file=p)
-        assert _delta._save_any_figs is True
+        assert len(_delta._save_fig_list) > 0
+        assert _delta._save_any_grids is False
+
+    def test_save_sedflux_figs(self, tmp_path):
+        p = utilities.yaml_from_dict(tmp_path, 'input.yaml',
+                                     {'save_sedflux_figs': True})
+        _delta = DeltaModel(input_file=p)
+        assert len(_delta._save_fig_list) > 0
+        assert _delta._save_any_grids is False
+
+    def test_save_sandfrac_figs(self, tmp_path):
+        p = utilities.yaml_from_dict(tmp_path, 'input.yaml',
+                                     {'save_sandfrac_figs': True})
+        _delta = DeltaModel(input_file=p)
+        assert len(_delta._save_fig_list) > 0
         assert _delta._save_any_grids is False
 
     def test_save_figs_sequential(self, tmp_path):
         p = utilities.yaml_from_dict(tmp_path, 'input.yaml',
                                      {'save_figs_sequential': False})
         _delta = DeltaModel(input_file=p)
-        assert _delta._save_any_figs is False
+        assert len(_delta._save_fig_list) == 0
         assert _delta._save_any_grids is False
         assert _delta._save_figs_sequential is False
 
@@ -616,6 +692,12 @@ class TestSettingParametersFromYAMLFile:
         assert np.any(_delta.sigma > 0)
         assert np.all(_delta.sigma <= (1e-9 * _delta.dt))
 
+    def test_sand_frac_bc(self, tmp_path):
+        p = utilities.yaml_from_dict(tmp_path, 'input.yaml',
+                                     {'sand_frac_bc': -1})
+        _delta = DeltaModel(input_file=p)
+        assert _delta.sand_frac_bc == -1
+
 
 class TestSettingOtherParametersFromYAMLSettings:
 
@@ -715,6 +797,15 @@ class TestSettingOtherParametersFromYAMLSettings:
         _delta = DeltaModel(input_file=p)
         assert _delta.CTR == 299  # 300th index
 
+    def test_CTR_small(self, tmp_path):
+        p = utilities.yaml_from_dict(tmp_path, 'input.yaml',
+                                     {'Length': 40,
+                                      'Width': 40,
+                                      'dx': 10})
+        _delta = DeltaModel(input_file=p)
+        # small case, instead of 4/2-1=1 it is 4/2=2
+        assert _delta.CTR == 2
+
     def test_gamma(self, tmp_path):
         p = utilities.yaml_from_dict(tmp_path, 'input.yaml',
                                      {'S0': 0.01,
@@ -860,3 +951,99 @@ class TestSettingOtherParametersFromYAMLSettings:
         assert _delta.dt == 625
         assert _delta.N_crossdiff == int(round(50000 / 50))
         assert _delta.diffusion_multiplier == (625 / 1000 * 0.3 * 0.5 / 5**2)
+
+    def test_active_layer_thickness_float(self, tmp_path):
+        p = utilities.yaml_from_dict(tmp_path, 'input.yaml',
+                                     {'active_layer_thickness': 2.7})
+        _delta = DeltaModel(input_file=p)
+        assert _delta.active_layer_thickness == 2.7
+
+    def test_active_layer_thickness_int(self, tmp_path):
+        p = utilities.yaml_from_dict(tmp_path, 'input.yaml',
+                                     {'active_layer_thickness': 2})
+        _delta = DeltaModel(input_file=p)
+        assert _delta.active_layer_thickness == 2
+
+    def test_active_layer_thickness_default(self, tmp_path):
+        p = utilities.yaml_from_dict(tmp_path, 'input.yaml')
+        _delta = DeltaModel(input_file=p)
+        assert _delta.active_layer_thickness == _delta.h0 / 2
+
+
+class TestInitMetadataList:
+
+    def test_save_list_exists(self, tmp_path):
+        file_name = 'user_parameters.yaml'
+        p, f = utilities.create_temporary_file(tmp_path, file_name)
+        utilities.write_parameter_to_file(f, 'out_dir', tmp_path / 'out_dir')
+        f.close()
+        delta = DeltaModel(input_file=p)
+        # check things about the metadata
+        assert hasattr(delta, '_save_var_list')
+        assert type(delta._save_var_list) == dict
+        assert 'meta' in delta._save_var_list.keys()
+        # save meta not on, so check that it is empty
+        assert delta._save_var_list['meta'] == {}
+
+    def test_default_meta_list(self, tmp_path):
+        file_name = 'user_parameters.yaml'
+        p, f = utilities.create_temporary_file(tmp_path, file_name)
+        utilities.write_parameter_to_file(f, 'out_dir', tmp_path / 'out_dir')
+        utilities.write_parameter_to_file(f, 'save_metadata', True)
+        f.close()
+        delta = DeltaModel(input_file=p)
+        # check things about the metadata
+        assert hasattr(delta, '_save_var_list')
+        assert type(delta._save_var_list) == dict
+        assert 'meta' in delta._save_var_list.keys()
+        # save meta on, so check that some expected values are there
+        assert 'L0' in delta._save_var_list['meta'].keys()
+        assert delta._save_var_list['meta']['L0'] == ['L0', 'cells', 'i8', ()]
+        assert 'H_SL' in delta._save_var_list['meta'].keys()
+        assert delta._save_var_list['meta']['H_SL'] == \
+            [None, 'meters', 'f4', 'total_time']
+
+    def test_netcdf_vars(self, tmp_path):
+        # test that stuff makes it to the netcdf file as expected
+        file_name = 'user_parameters.yaml'
+        p, f = utilities.create_temporary_file(tmp_path, file_name)
+        utilities.write_parameter_to_file(f, 'out_dir', tmp_path / 'out_dir')
+        utilities.write_parameter_to_file(f, 'save_eta_grids', True)
+        utilities.write_parameter_to_file(f, 'save_metadata', True)
+        f.close()
+        delta = DeltaModel(input_file=p)
+        # check things about the metadata
+        assert hasattr(delta, '_save_var_list')
+        assert type(delta._save_var_list) == dict
+        assert 'meta' in delta._save_var_list.keys()
+        # save meta on, so check that some expected values are there
+        assert 'L0' in delta._save_var_list['meta'].keys()
+        assert delta._save_var_list['meta']['L0'] == ['L0', 'cells', 'i8', ()]
+        assert 'H_SL' in delta._save_var_list['meta'].keys()
+        assert delta._save_var_list['meta']['H_SL'] == \
+            [None, 'meters', 'f4', 'total_time']
+        # check save var list for eta
+        assert 'eta' in delta._save_var_list.keys()
+        assert delta._save_var_list['eta'] == \
+            ['eta', 'meters', 'f4', ('total_time', 'length', 'width')]
+        # force save to netcdf
+        delta.save_grids_and_figs()
+        # close netcdf
+        delta.output_netcdf.close()
+        # check out the netcdf
+        data = Dataset(os.path.join(delta.prefix, 'pyDeltaRCM_output.nc'),
+                       'r+', format='NETCDF4')
+        # check for meta group
+        assert 'meta' in data.groups
+        # check for L0 a single value metadata
+        assert 'L0' in data['meta'].variables
+        assert data['meta']['L0'][0].data == delta.L0
+        # check H_SL a vector of metadata
+        assert 'H_SL' in data['meta'].variables
+        assert data['meta']['H_SL'].dimensions == ('total_time',)
+        assert data['time'].shape == data['meta']['H_SL'].shape
+        # check on the eta grid
+        assert 'eta' in data.variables
+        assert data['eta'].shape[0] == data['time'].shape[0]
+        assert data['eta'].shape[1] == delta.L
+        assert data['eta'].shape[2] == delta.W
