@@ -23,6 +23,8 @@ class Test__init__:
 
         assert _delta.time_iter == 0.
         assert _delta._is_finalized is False
+        # check that subclass parameter dictionary has been initialized
+        assert _delta.subclass_parameters == {}
 
     def test_error_if_no_file_found(self):
         with pytest.raises(FileNotFoundError):
@@ -266,6 +268,85 @@ class Test__init__:
         with pytest.raises(ValueError):
             _ = DeltaModel(input_file=p)
 
+    def test_negative_active_layer_thickness(self, tmp_path):
+        p = utilities.yaml_from_dict(tmp_path, 'input.yaml',
+                                     {'active_layer_thickness': -2.2})
+        with pytest.raises(ValueError):
+            _ = DeltaModel(input_file=p)
+
+    def test_badtype_figure_saving(self, tmp_path):
+        p = utilities.yaml_from_dict(tmp_path, 'input.yaml')
+        _model = DeltaModel(input_file=p, defer_output=True)
+        # add request to plot invalid attribute
+        _model._save_fig_list['beta'] = ['beta']
+        # try to finish init
+        _model.init_output_file()
+        with pytest.raises(AttributeError):
+            _model.output_data()
+
+    def test_badshape_figure_saving(self, tmp_path):
+        p = utilities.yaml_from_dict(tmp_path, 'input.yaml')
+        _model = DeltaModel(input_file=p, defer_output=True)
+        # add request to plot attribute w/ invalid shape
+        _model._save_fig_list['inlet'] = ['inlet']
+        # try to finish init
+        _model.init_output_file()
+        with pytest.raises(AttributeError):
+            _model.output_data()
+
+    def test_clobber_netcdf(self, tmp_path):
+        p = utilities.yaml_from_dict(tmp_path, 'input.yaml',
+                                     {'clobber_netcdf': True})
+        _model = DeltaModel(input_file=p)
+        # assert that model could have clobbered a netcdf
+        assert _model._clobber_netcdf is True
+
+    def test_clobbering(self, tmp_path):
+        p = utilities.yaml_from_dict(tmp_path, 'input.yaml',
+                                     {'clobber_netcdf': True,
+                                      'save_eta_grids': True})
+        _model_1 = DeltaModel(input_file=p)
+        _model_1.output_netcdf.close()
+        # assert that model could have clobbered a netcdf
+        assert _model_1._clobber_netcdf is True
+        # make a second model which clobbers, raising eyebrows (and warning)
+        with pytest.warns(UserWarning):
+            _model_2 = DeltaModel(input_file=p)
+        _model_2.output_netcdf.close()
+        assert _model_2._clobber_netcdf is True
+
+    def test_no_clobber_error(self, tmp_path):
+        p = utilities.yaml_from_dict(tmp_path, 'input.yaml',
+                                     {'save_eta_grids': True})
+        _model_1 = DeltaModel(input_file=p)
+        _model_1.output_netcdf.close()
+        # assert that model could not have clobbered a netcdf
+        assert _model_1._clobber_netcdf is False
+        # make a second model which raises error
+        with pytest.raises(FileExistsError):
+            _ = DeltaModel(input_file=p)
+
+
+class TestDeprecatedHooks:
+
+    class HookDeltaModel(DeltaModel):
+        """Dummy class to add old hook."""
+
+        def hook_sed_route(self):
+            """Old hook"""
+            pass
+
+    def test_if_hook_raise_error(self, tmp_path):
+        """Deprecated hooks cannot be implemented and will raise an error."""
+
+        # create a delta with default settings
+        p = utilities.yaml_from_dict(tmp_path, 'input.yaml')
+        with pytest.raises(AttributeError):
+            _ = self.HookDeltaModel(input_file=p)
+
+        # check that regular model still works though
+        _ = DeltaModel(input_file=p)
+
 
 class TestUpdate:
 
@@ -275,7 +356,7 @@ class TestUpdate:
         _delta = DeltaModel(input_file=p)
 
         # mock top-level methods, verify call was made to each
-        _delta.run_one_timestep = mock.MagicMock()
+        _delta.solve_water_and_sediment_timestep = mock.MagicMock()
         _delta.apply_subsidence = mock.MagicMock()
         _delta.finalize_timestep = mock.MagicMock()
         _delta.log_model_time = mock.MagicMock()
@@ -287,7 +368,7 @@ class TestUpdate:
         _delta.update()
 
         # assert calls
-        assert _delta.run_one_timestep.call_count == 1
+        assert _delta.solve_water_and_sediment_timestep.call_count == 1
         assert _delta.apply_subsidence.call_count == 1
         assert _delta.finalize_timestep.call_count == 1
         assert _delta.log_model_time.call_count == 1
@@ -303,7 +384,7 @@ class TestUpdate:
         _delta.update()
 
         # assert calls
-        assert _delta.run_one_timestep.call_count == 2
+        assert _delta.solve_water_and_sediment_timestep.call_count == 2
         assert _delta.apply_subsidence.call_count == 2
         assert _delta.finalize_timestep.call_count == 2
         assert _delta.log_model_time.call_count == 2
@@ -336,15 +417,13 @@ class TestFinalize:
         _delta.log_info = mock.MagicMock()
         _delta.output_data = mock.MagicMock()
         _delta.output_checkpoint = mock.MagicMock()
-        _delta.record_final_stratigraphy = mock.MagicMock()
 
         # run finalize
         _delta.finalize()
 
         # assert calls
         #  should hit all options since no saves
-        assert _delta.log_info.call_count == 2
-        assert _delta.record_final_stratigraphy.call_count == 1
+        assert _delta.log_info.call_count == 1
 
         # these were originally included in `finalize`, but no longer.
         #   the checks for no call are here to ensure we don't revert
@@ -399,7 +478,7 @@ class TestPublicSettersAndGetters:
         _delta = DeltaModel(input_file=p)
 
         # mock the init methods
-        _delta.create_other_variables = mock.MagicMock()
+        _delta.create_boundary_conditions = mock.MagicMock()
         _delta.init_sediment_routers = mock.MagicMock()
 
         # check initials
@@ -412,14 +491,14 @@ class TestPublicSettersAndGetters:
         assert _delta.channel_flow_velocity == _delta._u0
 
         # assert reinitializers called
-        assert (_delta.create_other_variables.called is True)
+        assert (_delta.create_boundary_conditions.called is True)
         assert (_delta.init_sediment_routers.called is True)
 
     def test_setting_getting_channel_width(self, tmp_path):
         p = utilities.yaml_from_dict(tmp_path, 'input.yaml')
         _delta = DeltaModel(input_file=p)
 
-        _delta.create_other_variables = mock.MagicMock()
+        _delta.create_boundary_conditions = mock.MagicMock()
         _delta.init_sediment_routers = mock.MagicMock()
 
         # check initials
@@ -427,14 +506,14 @@ class TestPublicSettersAndGetters:
 
         # change value
         #  the channel width is then changed internally with `N0`, according
-        #  to the `create_other_variables` so no change is actually made
+        #  to the `create_boundary_conditions` so no change is actually made
         #  here.
         with pytest.warns(UserWarning):
             _delta.channel_width = 300
         assert _delta.channel_width == 250  # not changed!
 
         # assert reinitializers called
-        assert (_delta.create_other_variables.called is True)
+        assert (_delta.create_boundary_conditions.called is True)
         assert (_delta.init_sediment_routers.called is True)
 
     def test_setting_getting_flow_depth(self, tmp_path):
@@ -442,7 +521,7 @@ class TestPublicSettersAndGetters:
         _delta = DeltaModel(input_file=p)
 
         # mock the init methods
-        _delta.create_other_variables = mock.MagicMock()
+        _delta.create_boundary_conditions = mock.MagicMock()
         _delta.init_sediment_routers = mock.MagicMock()
 
         # check initials
@@ -455,7 +534,7 @@ class TestPublicSettersAndGetters:
         assert _delta.channel_flow_depth == _delta._h0
 
         # assert reinitializers called
-        assert (_delta.create_other_variables.called is True)
+        assert (_delta.create_boundary_conditions.called is True)
         assert (_delta.init_sediment_routers.called is True)
 
     def test_setting_getting_influx_concentration(self, tmp_path):
@@ -463,7 +542,7 @@ class TestPublicSettersAndGetters:
         _delta = DeltaModel(input_file=p)
 
         # mock the init methods
-        _delta.create_other_variables = mock.MagicMock()
+        _delta.create_boundary_conditions = mock.MagicMock()
         _delta.init_sediment_routers = mock.MagicMock()
 
         # check initials
@@ -474,7 +553,7 @@ class TestPublicSettersAndGetters:
         assert _delta.C0_percent == 0.3
 
         # assert reinitializers called
-        assert (_delta.create_other_variables.called is True)
+        assert (_delta.create_boundary_conditions.called is True)
         assert (_delta.init_sediment_routers.called is True)
 
     def test_getter_nosetter_sea_surface_elevation(self, tmp_path):
