@@ -21,6 +21,31 @@ class TestPreprocessorSingleJobSetups:
         with pytest.raises(ValueError):
             _ = preprocessor.Preprocessor()
 
+    def test_py_hlvl_runjobs_simple_param(self, tmp_path):
+        # a single parameter
+        p = utilities.yaml_from_dict(tmp_path, 'input.yaml',
+                                     {'h0': 7.5})
+        pp = preprocessor.Preprocessor(p)
+
+        assert len(pp.file_list) == 1
+        assert pp._is_completed is False
+
+        # a parameter that takes null as the default
+        p = utilities.yaml_from_dict(tmp_path, 'input.yaml',
+                                     {'hb': 7.5})
+        pp = preprocessor.Preprocessor(p)
+
+        assert len(pp.file_list) == 1
+        assert pp._is_completed is False
+
+        # a parameter that takes null with null
+        p = utilities.yaml_from_dict(tmp_path, 'input.yaml',
+                                     {'hb': None})
+        pp = preprocessor.Preprocessor(p)
+
+        assert len(pp.file_list) == 1
+        assert pp._is_completed is False
+
     def test_py_hlvl_tsteps_yml_runjobs_sngle(self, tmp_path):
         p = utilities.yaml_from_dict(tmp_path, 'input.yaml',
                                      {'timesteps': 50})
@@ -186,6 +211,28 @@ class TestPreprocessorMatrixJobsSetups:
         assert sum([j == 0.6 for j in f_bedload_list]) == 1
         assert pp.config_dict['timesteps'] == 3
 
+    def test_py_hlvl_mtrx_smthng_null(self, tmp_path):
+        file_name = 'user_parameters.yaml'
+        p, f = utilities.create_temporary_file(tmp_path, file_name)
+        utilities.write_parameter_to_file(f, 'out_dir', tmp_path / 'test')
+        utilities.write_parameter_to_file(f, 'timesteps', 3)
+        utilities.write_matrix_to_file(f,
+                                       ['hb'],
+                                       [[None, 2, 7]])
+        f.close()
+        pp = preprocessor.Preprocessor(input_file=p)
+
+        assert pp._has_matrix is True
+        assert type(pp.file_list) is list
+        assert len(pp.file_list) == 3
+        assert pp._is_completed is False
+
+        hb_list = pp._matrix_table[:, 1]
+
+        assert sum([j == 'None' for j in hb_list]) == 1
+        assert sum([j == 2 for j in hb_list]) == 1
+        assert sum([j == 7 for j in hb_list]) == 1
+
     def test_py_hlvl_mtrx_2list(self, tmp_path):
         file_name = 'user_parameters.yaml'
         p, f = utilities.create_temporary_file(tmp_path, file_name)
@@ -528,6 +575,47 @@ class TestPreprocessorMatrixAndEnsembleJobsSetups:
             _ = preprocessor.Preprocessor(input_file=p, timesteps=3)
 
 
+class TestPreprocessorSetAndEnsembleJobsSetups:
+
+    def test_1_set_ensemble_3(self, tmp_path):
+        file_name = 'user_parameters.yaml'
+        p, f = utilities.create_temporary_file(tmp_path, file_name)
+        utilities.write_parameter_to_file(f, 'out_dir', tmp_path / 'test')
+        utilities.write_parameter_to_file(f, 'ensemble', 3)
+        utilities.write_set_to_file(f, [{'f_bedload': 0.77, 'u0': 1}])
+        f.close()
+        pp = preprocessor.Preprocessor(input_file=p, timesteps=3)
+
+        assert pp._has_set is True
+        assert pp._has_matrix is False
+        assert pp._has_ensemble is True
+        assert type(pp.file_list) is list
+        assert len(pp.file_list) == 3
+        assert pp._is_completed is False
+
+        f_bedload_list = pp._matrix_table[:, 1]
+
+        assert sum([j == 0.77 for j in f_bedload_list]) == 3
+        assert pp.config_dict['timesteps'] == 3
+
+
+class TestPreprocessorSetAndMatrixJobsSetups:
+
+    def test_not_possible(self, tmp_path):
+        file_name = 'user_parameters.yaml'
+        p, f = utilities.create_temporary_file(tmp_path, file_name)
+        utilities.write_parameter_to_file(f, 'out_dir', tmp_path / 'test')
+        utilities.write_set_to_file(f, [{'f_bedload': 0.77, 'h0': 1}])
+        utilities.write_matrix_to_file(f,
+                                       ['u0'],
+                                       [[0.2, 0.5, 0.6]])
+        f.close()
+
+        with pytest.raises(ValueError,
+                           match=r'Cannot use "matrix" and "set" .*'):
+            _ = preprocessor.Preprocessor(input_file=p, timesteps=3)
+
+
 class TestPreprocessorParallelJobsSetups:
     """
     Note that parallel job setup will work on any operating system. This is
@@ -812,7 +900,7 @@ class TestSerialJob:
         sj.deltamodel._checkpoint_dt = 2 * sj.deltamodel._dt
 
         # mock top-level methods, verify call was made to each
-        sj.deltamodel.run_one_timestep = mock.MagicMock()
+        sj.deltamodel.solve_water_and_sediment_timestep = mock.MagicMock()
         sj.deltamodel.apply_subsidence = mock.MagicMock()
         sj.deltamodel.finalize_timestep = mock.MagicMock()
         sj.deltamodel.log_model_time = mock.MagicMock()
@@ -827,7 +915,7 @@ class TestSerialJob:
         # assertions
         assert sj.deltamodel._time_iter == 10
         assert sj.deltamodel.output_data.call_count == 10
-        assert sj.deltamodel.run_one_timestep.call_count == 10
+        assert sj.deltamodel.solve_water_and_sediment_timestep.call_count == 10
         assert sj.deltamodel.apply_subsidence.call_count == 10
         assert sj.deltamodel.finalize_timestep.call_count == 10
         assert sj.deltamodel.log_model_time.call_count == 10
@@ -851,7 +939,7 @@ class TestSerialJob:
         sj.deltamodel._checkpoint_dt = 2 * sj.deltamodel._dt
 
         # mock top-level methods, verify call was made to each
-        sj.deltamodel.run_one_timestep = mock.MagicMock()
+        sj.deltamodel.solve_water_and_sediment_timestep = mock.MagicMock()
         sj.deltamodel.apply_subsidence = mock.MagicMock()
         sj.deltamodel.finalize_timestep = mock.MagicMock()
         sj.deltamodel.log_model_time = mock.MagicMock()
@@ -866,7 +954,7 @@ class TestSerialJob:
 
         # assertions
         assert sj.deltamodel._time_iter == 1
-        assert sj.deltamodel.run_one_timestep.call_count == 1
+        assert sj.deltamodel.solve_water_and_sediment_timestep.call_count == 1
         assert sj.deltamodel.apply_subsidence.call_count == 1
         assert sj.deltamodel.finalize_timestep.call_count == 1
         assert sj.deltamodel.log_model_time.call_count == 1
@@ -893,7 +981,7 @@ class TestSerialJob:
         sj.deltamodel._checkpoint_dt = 2 * sj.deltamodel._dt
 
         # mock top-level methods, verify call was made to each
-        sj.deltamodel.run_one_timestep = mock.MagicMock()
+        sj.deltamodel.solve_water_and_sediment_timestep = mock.MagicMock()
         sj.deltamodel.apply_subsidence = mock.MagicMock()
         sj.deltamodel.finalize_timestep = mock.MagicMock()
         sj.deltamodel.log_model_time = mock.MagicMock()
@@ -908,7 +996,7 @@ class TestSerialJob:
 
         # assertions
         assert sj.deltamodel._time_iter == 10
-        assert sj.deltamodel.run_one_timestep.call_count == 10
+        assert sj.deltamodel.solve_water_and_sediment_timestep.call_count == 10
         assert sj.deltamodel.apply_subsidence.call_count == 10
         assert sj.deltamodel.finalize_timestep.call_count == 10
         assert sj.deltamodel.log_model_time.call_count == 10
@@ -941,7 +1029,7 @@ class TestParallelJob:
         pj.deltamodel._checkpoint_dt = 2 * pj.deltamodel._dt
 
         # mock top-level methods, verify call was made to each
-        pj.deltamodel.run_one_timestep = mock.MagicMock()
+        pj.deltamodel.solve_water_and_sediment_timestep = mock.MagicMock()
         pj.deltamodel.apply_subsidence = mock.MagicMock()
         pj.deltamodel.finalize_timestep = mock.MagicMock()
         pj.deltamodel.log_model_time = mock.MagicMock()
@@ -950,18 +1038,29 @@ class TestParallelJob:
         pj.deltamodel.output_checkpoint = mock.MagicMock()
         pj.deltamodel.finalize = mock.MagicMock()
 
+        # mock initialization and their hooks
+        pj.deltamodel.init_output_file = mock.MagicMock()
+        pj.deltamodel.hook_init_output_file = mock.MagicMock()
+        pj.deltamodel.hook_output_data = mock.MagicMock()
+        pj.deltamodel.hook_output_checkpoint = mock.MagicMock()
+
         # run the method (call update 10 times)
         pj.run()
 
         # assertions
         assert pj.deltamodel._time_iter == 10
-        assert pj.deltamodel.run_one_timestep.call_count == 10
+        assert pj.deltamodel.solve_water_and_sediment_timestep.call_count == 10
         assert pj.deltamodel.apply_subsidence.call_count == 10
         assert pj.deltamodel.finalize_timestep.call_count == 10
         assert pj.deltamodel.log_model_time.call_count == 10
         assert pj.deltamodel.output_data.call_count == 11  # once in init
         assert pj.deltamodel.output_checkpoint.call_count == 11  # once in init
         assert pj.deltamodel.finalize.call_count == 1
+        # hooks/initialization
+        assert pj.deltamodel.init_output_file.call_count == 1
+        assert pj.deltamodel.hook_init_output_file.call_count == 1
+        assert pj.deltamodel.hook_output_data.call_count == 11
+        assert pj.deltamodel.hook_output_checkpoint.call_count == 11
 
         # check the log outputs for successes
         _calls = [mock.call({'job': 0, 'stage': 0, 'code': 0}),
@@ -984,7 +1083,7 @@ class TestParallelJob:
         pj.deltamodel._checkpoint_dt = 2 * pj.deltamodel._dt
 
         # mock top-level methods, verify call was made to each
-        pj.deltamodel.run_one_timestep = mock.MagicMock()
+        pj.deltamodel.solve_water_and_sediment_timestep = mock.MagicMock()
         pj.deltamodel.apply_subsidence = mock.MagicMock()
         pj.deltamodel.finalize_timestep = mock.MagicMock()
         pj.deltamodel.log_model_time = mock.MagicMock(
@@ -999,7 +1098,7 @@ class TestParallelJob:
 
         # assertions
         assert pj.deltamodel._time_iter == 1
-        assert pj.deltamodel.run_one_timestep.call_count == 1
+        assert pj.deltamodel.solve_water_and_sediment_timestep.call_count == 1
         assert pj.deltamodel.apply_subsidence.call_count == 1
         assert pj.deltamodel.finalize_timestep.call_count == 1
         assert pj.deltamodel.log_model_time.call_count == 1
@@ -1033,7 +1132,7 @@ class TestParallelJob:
         pj.deltamodel._checkpoint_dt = 2 * pj.deltamodel._dt
 
         # mock top-level methods, verify call was made to each
-        pj.deltamodel.run_one_timestep = mock.MagicMock()
+        pj.deltamodel.solve_water_and_sediment_timestep = mock.MagicMock()
         pj.deltamodel.apply_subsidence = mock.MagicMock()
         pj.deltamodel.finalize_timestep = mock.MagicMock()
         pj.deltamodel.log_model_time = mock.MagicMock()
@@ -1048,7 +1147,7 @@ class TestParallelJob:
 
         # assertions
         assert pj.deltamodel._time_iter == 10
-        assert pj.deltamodel.run_one_timestep.call_count == 10
+        assert pj.deltamodel.solve_water_and_sediment_timestep.call_count == 10
         assert pj.deltamodel.apply_subsidence.call_count == 10
         assert pj.deltamodel.finalize_timestep.call_count == 10
         assert pj.deltamodel.log_model_time.call_count == 10
@@ -1068,6 +1167,94 @@ class TestParallelJob:
         pj.deltamodel.logger.exception.assert_called_once()
 
 
+class TestWriteYamlConfigToFile:
+
+    def test_write_single_int(self, tmp_path):
+        # set up what to write
+        file_path = tmp_path / 'output.yml'
+        yaml_dict = {'variable': 1}
+
+        # write the file
+        preprocessor._write_yaml_config_to_file(yaml_dict, file_path)
+
+        with open(file_path) as f:
+            _returned = ' '.join(f.readlines())
+
+        assert _returned == 'variable: 1\n'
+
+    def test_write_single_float(self, tmp_path):
+        # set up what to write
+        file_path = tmp_path / 'output.yml'
+        yaml_dict = {'variable': 1.5}
+
+        # write the file
+        preprocessor._write_yaml_config_to_file(yaml_dict, file_path)
+
+        with open(file_path) as f:
+            _returned = ' '.join(f.readlines())
+
+        assert _returned == 'variable: 1.5\n'
+
+    def test_write_single_string(self, tmp_path):
+        # set up what to write
+        file_path = tmp_path / 'output.yml'
+        yaml_dict = {'variable': 'a string'}
+
+        # write the file
+        preprocessor._write_yaml_config_to_file(yaml_dict, file_path)
+
+        with open(file_path) as f:
+            _returned = ' '.join(f.readlines())
+
+        assert _returned == 'variable: a string\n'
+
+    def test_write_single_bool(self, tmp_path):
+        # set up what to write
+        file_path = tmp_path / 'output.yml'
+        yaml_dict = {'variable': False}
+
+        # write the file
+        preprocessor._write_yaml_config_to_file(yaml_dict, file_path)
+
+        with open(file_path) as f:
+            _returned = ' '.join(f.readlines())
+
+        assert _returned == 'variable: False\n'
+
+    def test_write_single_null(self, tmp_path):
+        # set up what to write
+        file_path = tmp_path / 'output.yml'
+        yaml_dict = {'variable': None}
+
+        # write the file
+        preprocessor._write_yaml_config_to_file(yaml_dict, file_path)
+
+        with open(file_path) as f:
+            _returned = ' '.join(f.readlines())
+
+        assert _returned == 'variable: null\n'
+
+    def test_write_multiple_one_each(self, tmp_path):
+        # set up what to write
+        file_path = tmp_path / 'output.yml'
+        yaml_dict = {'variable1': 1,
+                     'variable2': 2.2,
+                     'variable3': 'a string',
+                     'variable4': False,
+                     'variable5': None}
+
+        # write the file
+        preprocessor._write_yaml_config_to_file(yaml_dict, file_path)
+
+        with open(file_path) as f:
+            _returned = ' '.join(f.readlines())
+        print(_returned)
+
+        assert _returned == ('variable1: 1\n variable2: 2.2\n '
+                             'variable3: a string\n variable4: False\n '
+                             'variable5: null\n')
+
+
 class TestPreprocessorImported:
 
     def test_Preprocessor_toplevelimport(self):
